// DataObj.cpp : implementation file
//

#include "stdafx.h"
#include "Replacer.h"

#include "MainFrm.h"

#include "Sctipt.h"
#include "Filer.h"
#include "DataObj.h"

#include "ReplacerDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDataObj

void ShowOleErrorMsg(HRESULT hr, LPCTSTR lpszContextMsg);

// in reverse order:
COLUMN_PROP CDataObj::m_columns_prop[] =
{
	_T("Path"),		73,
	_T("End"),		73,
	_T("Begin"),	73,
	_T("File"),		73,
};

IMPLEMENT_DYNCREATE(CDataObj, CCmdTarget)

CDataObj::CDataObj() : m_bAutoDelNotExist(TRUE)
{
}

CDataObj::~CDataObj()
{
}


BEGIN_MESSAGE_MAP(CDataObj, CCmdTarget)
	//{{AFX_MSG_MAP(CDataObj)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDataObj message handlers

CDataObj::CDataObj(const CDataObj &dataObj)
{
	if (m_arrScripts.size())
		m_arrScripts.clear(); 
	
	m_arrScripts = dataObj.m_arrScripts;
	m_strDataFolder = dataObj.m_strDataFolder;
	m_strSourceFolder = dataObj.m_strSourceFolder;
	m_bSubFolders = dataObj.m_bSubFolders;
	m_bAutoDelNotExist = dataObj.m_bAutoDelNotExist;
	m_nCount = dataObj.m_nCount;
}

CDataObj& CDataObj::operator=(const CDataObj &dataObj)
{
	if (m_arrScripts.size()) 
		m_arrScripts.clear(); 
	
	m_arrScripts = dataObj.m_arrScripts;
	m_strDataFolder = dataObj.m_strDataFolder;
	m_strSourceFolder = dataObj.m_strSourceFolder;
	m_bSubFolders = dataObj.m_bSubFolders;
	m_bAutoDelNotExist = dataObj.m_bAutoDelNotExist;
	m_nCount = dataObj.m_nCount;
	
	return *this;
}

///////////////////////////////////////
// Read data from system regestry

BOOL CDataObj::ReadData()
{
	CWinApp* pApp = AfxGetApp();
	VERIFY(pApp != NULL);
	
	m_strDataFolder   = pApp->GetProfileString(_T("Folders"), _T("Data"));
	m_strSourceFolder = pApp->GetProfileString(_T("Folders"), _T("Source"), DEFAULT_EXT_LIST);
	m_nCount          = pApp->GetProfileInt(_T("Folders"), _T("Count"), 0);
	m_bSubFolders     = (BOOL)pApp->GetProfileInt(_T("Folders"), _T("IncludeSubfolders"), 1);

	if (m_arrScripts.size()) 
		m_arrScripts.clear(); 
	
	if (m_nCount)
	{
		// Create and read CSctipt objects
		_TCHAR szSection[MAX_LEN_SECTION];
		for (int nCount = 0; nCount < m_nCount; nCount++)
		{
			_stprintf(szSection, _T("%d"), nCount);
			m_arrScripts.push_back(CSctipt(pApp, szSection)); 
		}
	}
	return TRUE;
}

///////////////////////////////////////
// Save data to system regestry

BOOL CDataObj::SaveData()
{
	// Get count CSctipt objects
	int nCount = m_arrScripts.size(); // Get current counter

	// Save document data 
	CWinApp* pApp = AfxGetApp();
	VERIFY(pApp != NULL);
	
	int nOldCount = pApp->GetProfileInt(_T("Folders"), _T("Count"), 0); // Get ald counter
	m_nCount = nCount;													// Set current counter
	pApp->WriteProfileString(_T("Folders"), _T("Data"), m_strDataFolder);
	pApp->WriteProfileString(_T("Folders"), _T("Source"), m_strSourceFolder);
	pApp->WriteProfileInt(_T("Folders"), _T("Count"), m_nCount);
	pApp->WriteProfileInt(_T("Folders"), _T("IncludeSubfolders"), (int)m_bSubFolders);

	int nIndex = 0;
	_TCHAR szSection[MAX_LEN_SECTION];
	if (nCount)
	{
		// Save CSctipt objects
		vector<CSctipt>::iterator iter;
		for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
		{
			_stprintf(szSection, _T("%d"), nIndex++);
			(*iter).WriteData(pApp, szSection);
		}
	}

	if (nOldCount > nCount)
	{
		// Delete subkeys for CSctipt objects
		HKEY hKey;
		nIndex = nOldCount;
		LONG nCode = ERROR_SUCCESS; 
		CString strAppSubKey;
		CString strCurSubKey;
		strAppSubKey.Format(_T("Software\\%s\\%s"), pApp->m_pszRegistryKey, pApp->m_pszProfileName); 
		do
		{
			strCurSubKey.Format(_T("%s\\%d"), LPCTSTR(strAppSubKey), --nIndex);
			nCode = RegOpenKeyEx(
				HKEY_CURRENT_USER,
				LPCTSTR(strCurSubKey),
				NULL,
				KEY_READ|KEY_ENUMERATE_SUB_KEYS,
				&hKey);
			if (nCode == ERROR_SUCCESS)
			{
				RegCloseKey(hKey);
				nCode = RegDeleteKey(HKEY_CURRENT_USER,
					LPCTSTR(strCurSubKey));
			}
		}
		while(nIndex > nCount);
	}
	
	return TRUE;
}

///////////////////////////////////////
// Add colunms to ListView on PropPage

BOOL CDataObj::InitColumns(HWND hWndList)
{
	if (hWndList == NULL) return FALSE;
	
	// Insert columns
	LVCOLUMN col;
	memset(&col, NULL, sizeof(col));
	col.mask = LVCF_FMT|LVCF_SUBITEM|LVCF_TEXT|LVCF_WIDTH; 
	col.fmt = LVCFMT_LEFT;
	
	for (int iSubItem = 0; iSubItem < sizeof(m_columns_prop)/sizeof(COLUMN_PROP); iSubItem++)
	{
		col.iSubItem = iSubItem;
		col.pszText = (LPTSTR)m_columns_prop[iSubItem].lpszHeader;
		col.cx = m_columns_prop[iSubItem].nWidth;
		VERIFY(ListView_InsertColumn(hWndList, 0, &col) != -1);
	}
	return TRUE;
}

///////////////////////////////////////
// Fill contents ListView on PropPage

BOOL CDataObj::FillContents(HWND hWndList)
{
	if (hWndList == NULL) return FALSE;
	
	// Insert items
	LVITEM item;
	memset(&item, NULL, sizeof(item));
	item.mask = LVIF_TEXT;
	
	int nCount = m_arrScripts.size();
	if (nCount)
	{
		int nCurItem = 0;
		vector<CSctipt>::iterator iter;
		for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
		{
			item.iSubItem = 0;
			item.pszText = (LPTSTR)LPCTSTR((*iter).m_strFileName);
			VERIFY((nCurItem = ListView_InsertItem(hWndList, &item)) != -1);
			ListView_SetItemText(hWndList, nCurItem, 1, (LPTSTR)LPCTSTR((*iter).m_strBeginMark));
			ListView_SetItemText(hWndList, nCurItem, 2, (LPTSTR)LPCTSTR((*iter).m_strEndMark));
			ListView_SetItemText(hWndList, nCurItem, 3, (LPTSTR)LPCTSTR((*iter).m_strPath));
			nCurItem++;
		}
	}
	return TRUE;
}

///////////////////////////////////////
// Get script information

BOOL CDataObj::GetScriptInfo(LPCTSTR lpszFile, CString& strBegin, CString& strEnd)
{
	_TCHAR buf_begin[MAX_BUFF_LEN];
	_TCHAR buf_end[MAX_BUFF_LEN];
	if (m_filer.GetScriptInfo(lpszFile, buf_begin, MAX_BUFF_LEN, buf_end, MAX_BUFF_LEN))
	{
		strBegin = buf_begin;
		strEnd = buf_end;
		return TRUE;
	}
	else
		return FALSE;
}

////////////////////////////////////////////////
// Insert last element of vector into ListView

BOOL CDataObj::InsertBack(HWND hWndList)
{
	if (hWndList == NULL) return FALSE;
	
	// Insert items
	LVITEM item;
	memset(&item, NULL, sizeof(item));
	item.mask = LVIF_TEXT;
	
	int nCount = m_arrScripts.size();
	if (nCount)
	{
		int nCurItem = 0;
		CSctipt* pSctipt = &m_arrScripts.back(); 
		item.iSubItem = 0;
		item.pszText = (LPTSTR)LPCTSTR(pSctipt->m_strFileName);
		VERIFY((nCurItem = ListView_InsertItem(hWndList, &item)) != -1);
		ListView_SetItemText(hWndList, nCurItem, 1, (LPTSTR)LPCTSTR(pSctipt->m_strBeginMark));
		ListView_SetItemText(hWndList, nCurItem, 2, (LPTSTR)LPCTSTR(pSctipt->m_strEndMark));
		ListView_SetItemText(hWndList, nCurItem, 3, (LPTSTR)LPCTSTR(pSctipt->m_strPath));
		m_nCount++;
		return TRUE;
	}
	return FALSE;
}

/////////////////////////////////////////////////////////
// Remove selected element from ListView and from vector

BOOL CDataObj::DeleteSelected(HWND hWndList)
{
	if (hWndList == NULL) return FALSE;
	
	// Get selected item
	int nItem = -1;
	int nCount = ListView_GetItemCount(hWndList);
	if (nCount)
		if (ListView_GetSelectedCount(hWndList))
			nItem = ListView_GetNextItem(hWndList, -1, LVNI_SELECTED);
	
	if (nItem != -1)
	{
		// Get item info
		_TCHAR szPath[_MAX_PATH];
		_TCHAR szFileName[_MAX_PATH];
		ListView_GetItemText(hWndList, nItem, 0, szFileName, _MAX_PATH);
		ListView_GetItemText(hWndList, nItem, 3, szPath, _MAX_PATH);

		// Delete item
		BOOL bIsErase = FALSE;
		if (ListView_DeleteItem(hWndList, nItem))
		{
			// ... and erase from vector
			vector<CSctipt>::iterator iter;
			for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
			{
				if (_tcscmp((*iter).m_strFileName, szFileName) == 0 && 
					_tcscmp((*iter).m_strPath, szPath) == 0)
				{
					m_arrScripts.erase(iter);
					bIsErase = TRUE;
					m_nCount--;
					break;
				}
			}
		}
		return bIsErase;
	}
	return FALSE;
}

/////////////////////////////////////////////////////////
// Replace selected element in ListView and in vector

BOOL CDataObj::ReplaceSelected(HWND hWndList,
	const CString strFileName, const CString strBegine,
	const CString strEnd, const CString strPath)
{
	if (hWndList == NULL) return FALSE;
	
	// Get selected item
	int nItem = -1;
	int nCount = ListView_GetItemCount(hWndList);
	if (nCount)
		if (ListView_GetSelectedCount(hWndList))
			nItem = ListView_GetNextItem(hWndList, -1, LVNI_SELECTED);
	
	if (nItem != -1)
	{
		// Get item info
		_TCHAR szPath[_MAX_PATH];
		_TCHAR szFileName[_MAX_PATH];
		ListView_GetItemText(hWndList, nItem, 0, szFileName, _MAX_PATH);
		ListView_GetItemText(hWndList, nItem, 3, szPath, _MAX_PATH);

		BOOL bFound = FALSE;
		vector<CSctipt>::iterator iter;
		for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
		{
			if (_tcscmp((*iter).m_strFileName, szFileName) == 0 && 
				_tcscmp((*iter).m_strPath, szPath) == 0)
			{
				(*iter).m_strFileName = strFileName;
				(*iter).m_strBeginMark = strBegine;
				(*iter).m_strEndMark = strEnd;
				(*iter).m_strPath = strPath;
				ListView_SetItemText(hWndList, nItem, 0, (LPTSTR)LPCTSTR((*iter).m_strFileName));
				ListView_SetItemText(hWndList, nItem, 1, (LPTSTR)LPCTSTR((*iter).m_strBeginMark));
				ListView_SetItemText(hWndList, nItem, 2, (LPTSTR)LPCTSTR((*iter).m_strEndMark));
				ListView_SetItemText(hWndList, nItem, 3, (LPTSTR)LPCTSTR((*iter).m_strPath));
				bFound = TRUE;
				break;
			}
		}
		return bFound;
	}
	return FALSE;
}

////////////////////////////////////////////////
// Add new Item into ListView

BOOL CDataObj::Add(HWND hWndList, CString& strFileName)
{
	if (hWndList == NULL) return FALSE;
	
	BOOL bRetVal = FALSE;
	CString strBegine;
	CString strEnd;
	if (GetScriptInfo(strFileName, strBegine, strEnd))
	{
		CString strPath;
		int nEndBS = strFileName.ReverseFind('\\'); 
		if (nEndBS != -1)
		{
			strPath = strFileName.Left(++nEndBS); 
			strFileName = strFileName.Mid(nEndBS); 	
		}
		
		m_arrScripts.push_back(CSctipt(strFileName,
			strBegine, strEnd, strPath));
		bRetVal = InsertBack(hWndList);
	}
	return bRetVal;
}

////////////////////////////////////////////////
// Replace selected Item in ListView

BOOL CDataObj::Replace(HWND hWndList, CString &strFileName)
{
	if (hWndList == NULL) return FALSE;
	
	BOOL bRetVal = FALSE;
	CString strBegine;
	CString strEnd;
	if (GetScriptInfo(strFileName, strBegine, strEnd))
	{
		CString strPath;
		int nEndBS = strFileName.ReverseFind('\\'); 
		if (nEndBS != -1)
		{
			strPath = strFileName.Left(++nEndBS); 
			strFileName = strFileName.Mid(nEndBS); 	
		}
		bRetVal = ReplaceSelected(hWndList,
				strFileName, strBegine, strEnd, strPath);
	}
	return bRetVal;
}

///////////////////////////////////////////////////////////////
// Update all script files information (all Items in ListView)

BOOL CDataObj::UpdateAll(HWND hWndList)
{
	if (hWndList == NULL) return FALSE;
	
	BOOL bFound = FALSE;
	int nCount = ListView_GetItemCount(hWndList);
	if (nCount)
	{
		_TCHAR szPath[_MAX_PATH];
		_TCHAR szFileName[_MAX_PATH];
		_TCHAR szBegin[_MAX_PATH];
		_TCHAR szEnd[_MAX_PATH];
		CString strFileName;
		CString strBegine;
		CString strEnd;
		vector<int> arrNotExist;
		vector<CSctipt>::iterator iter;
		int nItem = -1;
		for (nItem = 0; nItem < nCount; nItem++)
		{
			// Get script info from ListView 
			ListView_GetItemText(hWndList, nItem, 0, szFileName, _MAX_PATH);
			ListView_GetItemText(hWndList, nItem, 1, szBegin, _MAX_PATH);
			ListView_GetItemText(hWndList, nItem, 2, szEnd, _MAX_PATH);
			ListView_GetItemText(hWndList, nItem, 3, szPath, _MAX_PATH);
			
			// Get script filename 
			strFileName = szPath;
			strFileName += szFileName;
			
			// Chech file on disk
			if (m_filer.IsFile(LPCTSTR(strFileName)))
			{
				// Get script info from file 
				if (GetScriptInfo(strFileName, strBegine, strEnd) &&
					(_tcscmp(LPCTSTR(strBegine), szBegin) != 0
					|| _tcscmp(LPCTSTR(strEnd), szEnd) != 0))
				{	
					// Need change strBegine & strEnd
					for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
					{
						if (_tcscmp((*iter).m_strFileName, szFileName) == 0 && 
							_tcscmp((*iter).m_strPath, szPath) == 0)
						{
							(*iter).m_strBeginMark = strBegine;
							(*iter).m_strEndMark = strEnd;
							ListView_SetItemText(hWndList, nItem, 1, (LPTSTR)LPCTSTR((*iter).m_strBeginMark));
							ListView_SetItemText(hWndList, nItem, 2, (LPTSTR)LPCTSTR((*iter).m_strEndMark));
							bFound = TRUE;
							break;
						}
					}
				}
			}
			else
			{
				// File not exist
				if (m_bAutoDelNotExist)
				{
					// store it into vector arrNotExist
					arrNotExist.push_back(nItem); 
				}
			}
		}
		int nDeleted = arrNotExist.size();
		if (nDeleted)
		{
			// Delete all not exist files
			::MessageBeep((DWORD)(-1));
			if (::MessageBox(NULL, 
				_T("Found one or more not existing scripts file. Are your wish to delete all such files from the list?"),
				_T("Confirm deleting files"), MB_YESNO|MB_ICONQUESTION) == IDYES)
			{
				vector<int>::reverse_iterator riter;
				for (riter = arrNotExist.rbegin(); riter != arrNotExist.rend(); ++riter)
				{
					// Get sctipt for delete
					nItem = (*riter);
					ListView_GetItemText(hWndList, nItem, 0, szFileName, _MAX_PATH);
					ListView_GetItemText(hWndList, nItem, 3, szPath, _MAX_PATH);
					
					// Delete from ListView
					VERIFY(ListView_DeleteItem(hWndList, nItem));

					// ... and delete from m_arrScripts
					for(iter = m_arrScripts.begin(); iter != m_arrScripts.end(); iter++)   
					{
						if (_tcscmp((*iter).m_strFileName, szFileName) == 0 && 
							_tcscmp((*iter).m_strPath, szPath) == 0)
						{
							m_arrScripts.erase(iter); 
							bFound = TRUE;
							break;
						}
					}
				}
				m_nCount -= nDeleted;
			}
		}
	}
	return bFound;
}

//////////////////////////////
// Update all source files

BOOL CDataObj::UpdateSource()
{
	CMainFrame* pWnd = (CMainFrame*)AfxGetMainWnd();
	VERIFY(pWnd != NULL);
	CReplacerDoc* pDoc = (CReplacerDoc*)pWnd->GetActiveDocument();
	VERIFY(pDoc != NULL);

	if (m_arrScripts.size() == 0)
	{
		return FALSE;
	}
	if (m_strSourceFolder.GetLength() == 0)
	{
		return FALSE;
	}
	if (m_strDataFolder.GetLength() == 0)
	{
		return FALSE;
	}
	
	BOOL bNoErr = FALSE;
	m_nFiles = 0;
	m_nAllFiles = 0;

	if (pDoc != NULL)
		pDoc->PrepUpdateSource();
	bNoErr = UpdateInFolder(pWnd, pDoc, LPCTSTR(m_strSourceFolder), TRUE);
	if (bNoErr && m_nFiles > 0)
	{
		m_nAllFiles = m_nFiles;
		m_nFiles = 0;
		if (pWnd != NULL)
			pWnd->SendMessage(WM_USER_PROGRESS_START,(WPARAM)m_nAllFiles); 
		bNoErr = UpdateInFolder(pWnd, pDoc, LPCTSTR(m_strSourceFolder));
		if (pWnd != NULL)
			pWnd->SendMessage(WM_USER_PROGRESS_END); 
	}
	return bNoErr;
}

BOOL CDataObj::UpdateInFolder(CMainFrame* pWnd, CReplacerDoc* pDoc, LPCTSTR lpszSourceFolder, BOOL bCountOnly)
{
	BOOL bRetVal = FALSE;
	CString strFilter(lpszSourceFolder);
	ULONG nCount = 0;
	LPCTSTR lpextp = NULL;
	LPCTSTR lpextlst = LPCTSTR(m_strDataFolder);
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;
	
	strFilter += _T("*");
	hFind = FindFirstFile(LPCTSTR(strFilter), &FindFileData);

	if (hFind == INVALID_HANDLE_VALUE)
	{
		ShowOleErrorMsg((HRESULT)GetLastError(), _T("FindFirstFile"));
		return bRetVal;
	}

	vector<CString> arrSubDirs;
	bRetVal = TRUE;

	do
	{
		if (FindFileData.dwFileAttributes == 0xffffffff)
			continue;
		if (FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (!(_tcscmp(FindFileData.cFileName, _T(".")) == 0
				|| _tcscmp(FindFileData.cFileName, _T("..")) == 0))
			{
				// it's subdir
				if (m_bSubFolders)
					arrSubDirs.push_back(FindFileData.cFileName); 
			}
			continue;
		}
		if (FindFileData.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE ||
			FindFileData.dwFileAttributes & FILE_ATTRIBUTE_NORMAL)
		{
			// it's file -> is extention?
			if ((lpextp = _tcsrchr(FindFileData.cFileName,'.')) != NULL)
			{
				// Check filter extensions
				if (_tcsstr(lpextlst ,lpextp+1) != NULL)
				{
					if (!bCountOnly)
					{
						if (!UpdateFile(pWnd, pDoc, lpszSourceFolder, FindFileData.cFileName))
							bRetVal = FALSE;
						pWnd->SendMessage(WM_USER_PROGRESS_SHOW); 
					}
					m_nFiles++;
				}
			}
		}
	}
	while (FindNextFile(hFind, &FindFileData));
	
	FindClose(hFind);
	
	if (arrSubDirs.size())
	{
		// Update all subdirs
		CString strSubDir;
		vector<CString>::iterator iter;
		for(iter = arrSubDirs.begin(); iter != arrSubDirs.end(); iter++)
		{
			strSubDir = lpszSourceFolder;
			strSubDir += (*iter);
			strSubDir += _T("\\");
			if (!UpdateInFolder(pWnd, pDoc, LPCTSTR(strSubDir), bCountOnly))
				bRetVal = FALSE;
		}
	}
	
	return bRetVal;
}

BOOL CDataObj::UpdateFile(CMainFrame* pWnd, CReplacerDoc* pDoc, LPCTSTR lpszSourceFolder, LPCTSTR lpszFileName)
{
	_TCHAR szFileName[_MAX_PATH];
	_tcscpy(szFileName, lpszSourceFolder);
	_tcscat(szFileName, lpszFileName);
	if (pDoc != NULL)
		pDoc->AddSource(lpszSourceFolder, lpszFileName);
	return m_filer.UpdateFile(pWnd, pDoc, szFileName, &m_arrScripts);
}
